package com.hyc.consumer;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.hyc.config.RabbitmqConfig;
import com.hyc.domain.AnalyzeRule;
import com.hyc.domain.ConnectionParameter;
import com.hyc.util.ConversionUtil;
import lombok.extern.slf4j.Slf4j;
import org.eclipse.paho.client.mqttv3.*;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.Cache;
import org.springframework.cache.CacheManager;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.stereotype.Component;
import org.springframework.transaction.support.TransactionTemplate;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.UUID;

/**
 * 创建MQTT客户端消费者
 *
 * @author YouChe·He
 * @ClassName: CreateMqttClientConsumer
 * @Description: 创建MQTT客户端
 * @CreateTime: 2024/6/18 15:45
 */
@Component
@Slf4j
public class CreateMqttClientConsumer {

    @Autowired
    private TransactionTemplate transactionTemplate;


    @Autowired
    private RedisTemplate<String, String> redisTemplate;


    @Autowired
    private KafkaTemplate<Object, Object> kafkaTemplate;
    @Autowired
    private CacheManager cacheManager;


    @RabbitListener(queues = RabbitmqConfig.CREATE_MQTT_CLIENT)
    public void createMqttClient(String connectionParameterJson) {
        log.info("消息是:{}", connectionParameterJson);
        String s = connectionParameterJson.replaceAll("\\\\", "");
        String substring = s.substring(1, s.length() - 1);
        log.error("切割之后:{}", substring);
        ConnectionParameter connectionParameter = JSONObject.parseObject(substring, ConnectionParameter.class);

        boolean oneMqttClient = createOneMqttClient(connectionParameter);
        if (oneMqttClient == true) {
            log.warn("broker为:{}的mqtt客户端连接成功", connectionParameter.getBroker());
        }
    }


    public boolean createOneMqttClient(ConnectionParameter connectionParameter) {

        int qos = 0;
        try {
            MqttClient client = null;

            client = new MqttClient(connectionParameter.getBroker(), connectionParameter.getClientid(), new MemoryPersistence());


            // 连接参数
            MqttConnectOptions options = new MqttConnectOptions();
            if (connectionParameter.getUsername() != null && !"".equals(connectionParameter.getUsername())) {
                options.setUserName(connectionParameter.getUsername());
                options.setPassword(connectionParameter.getPassword().toCharArray());
            }

            options.setConnectionTimeout(60);
            options.setKeepAliveInterval(60);
            // 设置回调
            client.setCallback(new MqttCallback() {

                @Override
                public void connectionLost(Throwable cause) {
                    cause.printStackTrace();
                    System.out.println("connectionLost: ");
                }

                @Override
                public void messageArrived(String topic, MqttMessage message) {


                    log.info("topic: {}", topic);
                    ;
                    String resultString = ConversionUtil.hexStringToString(new String(message.getPayload()));
                    resultString = resultString.substring(1, 208);
                    log.warn("解析后的字符串是:{}", resultString);
                    log.warn("长度为:{}", resultString.length());

                    String vin = resultString.substring(0, 17);
                    log.error("vins:{}", vin);

                    Cache analyzeRuleCache = cacheManager.getCache("analyzeRule");
                    Cache.ValueWrapper valueWrapper = analyzeRuleCache.get(vin + "_Analyze_Rule");
                    String cacheRuleString = null;
                    if (valueWrapper != null) {
                        cacheRuleString = valueWrapper.get().toString();
                    } else {
                        cacheRuleString = redisTemplate.opsForValue().get(vin + "_Analyze_Rule");
                    }

                    List<AnalyzeRule> analyzeRules = JSONObject.parseArray(cacheRuleString, AnalyzeRule.class);
                    log.error("规则长度是:{}", analyzeRules.size());
                    LinkedHashMap<String, Object> linkedHashMap = new LinkedHashMap<>();
                    for (AnalyzeRule analyzeRule : analyzeRules) {
                        linkedHashMap.put(analyzeRule.getFieldsName(), resultString.substring(analyzeRule.getStartIndex(), analyzeRule.getEndIndex()));
                    }

                    log.warn("hashMap:{}", linkedHashMap);
                    ObjectMapper objectMapper = new ObjectMapper();
                    try {
                        String jsonData = objectMapper.writeValueAsString(linkedHashMap);
                        log.error("json格式:{}", jsonData);
                        String finalVin = vin;
                        transactionTemplate.execute(status -> {
                            try {
                                kafkaTemplate.send("topichyc", finalVin, jsonData);
                                status.flush();
                            } catch (Exception e) {
                                log.error("kafka生产异常: {}", e.getMessage());
                                status.setRollbackOnly();
                            }
                            return null;
                        });

                    } catch (JsonProcessingException e) {
                        e.printStackTrace();
                    }


                }

                @Override
                public void deliveryComplete(IMqttDeliveryToken token) {
                    System.out.println("deliveryComplete---------" + token.isComplete());
                }

            });
            client.connect(options);
            client.subscribe(connectionParameter.getTopic(), qos);
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
        return true;
    }

}
